@@ -13,7 +13,7 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2021 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2024 Live Networks, Inc.  All rights reserved.
 // Basic Usage Environment: for a simple, non-scripted, console application
 // Implementation
 
@@ -177,35 +177,32 @@ void BasicTaskScheduler::SingleStep(unsigned maxDelayTime) {
 
   // Also handle any newly-triggered event (Note that we do this *after* calling a socket handler,
   // in case the triggered event handler modifies The set of readable sockets.)
-  if (fTriggersAwaitingHandling != 0) {
-    if (fTriggersAwaitingHandling == fLastUsedTriggerMask) {
-      // Common-case optimization for a single event trigger:
-      fTriggersAwaitingHandling &=~ fLastUsedTriggerMask;
-      if (fTriggeredEventHandlers[fLastUsedTriggerNum] != NULL) {
-	(*fTriggeredEventHandlers[fLastUsedTriggerNum])(fTriggeredEventClientDatas[fLastUsedTriggerNum]);
-      }
-    } else {
-      // Look for an event trigger that needs handling (making sure that we make forward progress through all possible triggers):
-      unsigned i = fLastUsedTriggerNum;
-      EventTriggerId mask = fLastUsedTriggerMask;
+  if (fEventTriggersAreBeingUsed) {
+    // Look for an event trigger that needs handling (making sure that we make forward progress through all possible triggers):
+    unsigned i = fLastUsedTriggerNum;
+    EventTriggerId mask = fLastUsedTriggerMask;
 
-      do {
-	i = (i+1)%MAX_NUM_EVENT_TRIGGERS;
-	mask >>= 1;
-	if (mask == 0) mask = 0x80000000;
+    do {
+      i = (i+1)%MAX_NUM_EVENT_TRIGGERS;
+      mask >>= 1;
+      if (mask == 0) mask = EVENT_TRIGGER_ID_HIGH_BIT;
 
-	if ((fTriggersAwaitingHandling&mask) != 0) {
-	  fTriggersAwaitingHandling &=~ mask;
-	  if (fTriggeredEventHandlers[i] != NULL) {
-	    (*fTriggeredEventHandlers[i])(fTriggeredEventClientDatas[i]);
-	  }
-
-	  fLastUsedTriggerMask = mask;
-	  fLastUsedTriggerNum = i;
-	  break;
+#ifndef NO_STD_LIB
+      if (fTriggersAwaitingHandling[i].test()) {
+	fTriggersAwaitingHandling[i].clear();
+#else
+      if (fTriggersAwaitingHandling[i]) {
+	fTriggersAwaitingHandling[i] = False;
+#endif
+	if (fTriggeredEventHandlers[i] != NULL) {
+	  (*fTriggeredEventHandlers[i])(fTriggeredEventClientDatas[i]);
 	}
-      } while (i != fLastUsedTriggerNum);
-    }
+
+	fLastUsedTriggerMask = mask;
+	fLastUsedTriggerNum = i;
+	break;
+      }
+    } while (i != fLastUsedTriggerNum);
   }
 
   // Also handle any delayed event that may have come due.
