@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "groupsock"
-// Copyright (c) 1996-2021 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2024 Live Networks, Inc.  All rights reserved.
 // Network Addresses
 // Implementation
 
@@ -23,13 +23,10 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 #include <stddef.h>
 #include <stdio.h>
-#if defined(__WIN32__) || defined(_WIN32)
-#define USE_GETHOSTBYNAME 1 /*because at least some Windows don't have getaddrinfo()*/
-#else
+
 #ifndef INADDR_NONE
 #define INADDR_NONE 0xFFFFFFFF
 #endif
-#endif
 
 ////////// NetAddress //////////
 
