@@ -59,6 +59,21 @@ OnDemandServerMediaSubsession::~OnDemandServerMediaSubsession() {
 
 char const*
 OnDemandServerMediaSubsession::sdpLines(int addressFamily) {
+  if (fLastStreamToken != NULL && fReuseFirstSource) {
+    // We're reusing an existing stream.
+    // Hack: Check whether the SRTP ROC has changed.
+    // If so, we need to regenerate the SDP description.
+    RTPSink* rtpSink = ((StreamState*)fLastStreamToken)->rtpSink();
+    if (rtpSink != NULL && rtpSink->srtpROC() != fSRTP_ROC) {
+      fSRTP_ROC = rtpSink->srtpROC();
+      rtpSink->setupForSRTP(fParentSession->streamingIsEncrypted, fSRTP_ROC);
+      setSDPLinesFromRTPSink(rtpSink, getStreamSource(fLastStreamToken), rtpSink->estimatedBitrate());
+
+      RTCPInstance* rtcp = ((StreamState*)fLastStreamToken)->rtcpInstance();
+      if (rtcp != NULL) rtcp->setupForSRTCP();
+    }
+  }
+
   if (fSDPLines == NULL) {
     // We need to construct a set of SDP lines that describe this
     // subsession (as a unicast stream).  To do so, we first create
@@ -73,8 +88,15 @@ OnDemandServerMediaSubsession::sdpLines(int addressFamily) {
     RTPSink* dummyRTPSink = createNewRTPSink(dummyGroupsock, rtpPayloadType, inputSource);
     if (dummyRTPSink != NULL) {
       if (fParentSession->streamingUsesSRTP) {
-	fMIKEYStateMessage = dummyRTPSink->setupForSRTP(fParentSession->streamingIsEncrypted,
-							fMIKEYStateMessageSize);
+	if (fMIKEYStateMessage != NULL) {
+	  // Use the existing stream's MIKEY info to generate the SDP:
+	  dummyRTPSink->setupForSRTP(fMIKEYStateMessage, fMIKEYStateMessageSize, fSRTP_ROC);
+	} else {
+	  // Create new MIKEY info for this stream:
+	  fMIKEYStateMessage
+	    = dummyRTPSink->setupForSRTP(fParentSession->streamingIsEncrypted, fSRTP_ROC,
+					 fMIKEYStateMessageSize);
+	}
       }
 
       if (dummyRTPSink->estimatedBitrate() > 0) estBitrate = dummyRTPSink->estimatedBitrate();
@@ -177,7 +199,7 @@ void OnDemandServerMediaSubsession
 	  : createNewRTPSink(rtpGroupsock, rtpPayloadType, mediaSource);
 	if (rtpSink != NULL) {
 	  if (fParentSession->streamingUsesSRTP) {
-	    rtpSink->setupForSRTP(fMIKEYStateMessage, fMIKEYStateMessageSize);
+	    rtpSink->setupForSRTP(fMIKEYStateMessage, fMIKEYStateMessageSize, fSRTP_ROC);
 	  }
 	  if (rtpSink->estimatedBitrate() > 0) streamBitrate = rtpSink->estimatedBitrate();
 	}
@@ -436,7 +458,7 @@ void OnDemandServerMediaSubsession
   char const* mediaType = rtpSink->sdpMediaType();
   unsigned char rtpPayloadType = rtpSink->rtpPayloadType();
   struct sockaddr_storage const& addressForSDP = rtpSink->groupsockBeingUsed().groupAddress();
-  portNumBits portNumForSDP = ntohs(rtpSink->groupsockBeingUsed().port().num());
+  portNumBits portNumForSDP = 0; //ntohs(rtpSink->groupsockBeingUsed().port().num());
 
   AddressString ipAddressStr(addressForSDP);
   char* rtpmapLine = rtpSink->rtpmapLine();
@@ -571,13 +593,17 @@ void StreamState
   }
 
   if (!fAreCurrentlyPlaying && fMediaSource != NULL) {
-    if (fRTPSink != NULL) {
-      fRTPSink->startPlaying(*fMediaSource, afterPlayingStreamState, this);
-      fAreCurrentlyPlaying = True;
-    } else if (fUDPSink != NULL) {
-      fUDPSink->startPlaying(*fMediaSource, afterPlayingStreamState, this);
-      fAreCurrentlyPlaying = True;
+    MediaSink* sink;
+    if (fRTPSink != NULL) { sink = fRTPSink; }
+    else if (fUDPSink != NULL) { sink = fUDPSink; }
+    else return;
+    
+    if (!sink->startPlaying(*fMediaSource, afterPlayingStreamState, this)) {
+      fMediaSource->envir() << "sink->startPlaying() failed: "
+			    << fMediaSource->envir().getResultMsg() << "\n";
+      return;
     }
+    fAreCurrentlyPlaying = True;
   }
 }
 
@@ -604,9 +630,6 @@ void StreamState::endPlaying(Destinations* dests, unsigned clientSessionId) {
 
   if (dests->isTCP) {
     if (fRTPSink != NULL) {
-      // Comment out the following, because it prevents the "RTSPClientConnection" object
-      // from being closed after handling a "TEARDOWN": #####
-      //RTPInterface::clearServerRequestAlternativeByteHandler(fRTPSink->envir(), dests->tcpSocketNum);
       fRTPSink->removeStreamSocket(dests->tcpSocketNum, dests->rtpChannelId);
     }
     if (fRTCPInstance != NULL) {
