@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2021 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2024 Live Networks, Inc.  All rights reserved.
 // An abstraction of a network interface used for RTP (or RTCP).
 // (This allows the RTP-over-TCP hack (RFC 2326, section 10.12) to
 // be implemented transparently.)
@@ -382,7 +382,7 @@ Boolean RTPInterface::sendDataOverTCP(int socketNum, TLSState* tlsState,
 				      Boolean forceSendToSucceed) {
   int sendResult = (tlsState != NULL && tlsState->isNeeded)
     ? tlsState->write((char const*)data, dataSize)
-    : send(socketNum, (char const*)data, dataSize, 0/*flags*/);
+    : send(socketNum, (char const*)data, dataSize, MSG_NOSIGNAL/*flags*/);
   if (sendResult < (int)dataSize) {
     // The TCP send() failed - at least partially.
 
@@ -398,7 +398,8 @@ Boolean RTPInterface::sendDataOverTCP(int socketNum, TLSState* tlsState,
       makeSocketBlocking(socketNum, RTPINTERFACE_BLOCKING_WRITE_TIMEOUT_MS);
       sendResult = (tlsState != NULL && tlsState->isNeeded)
 	? tlsState->write((char const*)(&data[numBytesSentSoFar]), numBytesRemainingToSend)
-	: send(socketNum, (char const*)(&data[numBytesSentSoFar]), numBytesRemainingToSend, 0/*flags*/);
+	: send(socketNum, (char const*)(&data[numBytesSentSoFar]), numBytesRemainingToSend, MSG_NOSIGNAL/*flags*/);
+      makeSocketNonBlocking(socketNum);
       if ((unsigned)sendResult != numBytesRemainingToSend) {
 	// The blocking "send()" failed, or timed out.  In either case, we assume that the
 	// TCP connection has failed (or is 'hanging' indefinitely), and we stop using it
@@ -411,7 +412,6 @@ Boolean RTPInterface::sendDataOverTCP(int socketNum, TLSState* tlsState,
 	removeStreamSocket(socketNum, 0xFF);
 	return False;
       }
-      makeSocketNonBlocking(socketNum);
 
       return True;
     } else if (sendResult < 0 && envir().getErrno() != EAGAIN) {
@@ -434,6 +434,7 @@ SocketDescriptor::SocketDescriptor(UsageEnvironment& env, int socketNum, TLSStat
 }
 
 SocketDescriptor::~SocketDescriptor() {
+  fDeleteMyselfNext = False;
   fEnv.taskScheduler().turnOffBackgroundReadHandling(fOurSocketNum);
   removeSocketDescription(fEnv, fOurSocketNum);
 
@@ -510,10 +511,14 @@ void SocketDescriptor
 void SocketDescriptor::tcpReadHandler(SocketDescriptor* socketDescriptor, int mask) {
   // Call the read handler until it returns false, with a limit to avoid starving other sockets
   unsigned count = 2000;
+  Boolean areInRecursiveCall = socketDescriptor->fAreInReadHandlerLoop;
+
   socketDescriptor->fAreInReadHandlerLoop = True;
   while (!socketDescriptor->fDeleteMyselfNext && socketDescriptor->tcpReadHandler1(mask) && --count > 0) {}
-  socketDescriptor->fAreInReadHandlerLoop = False;
-  if (socketDescriptor->fDeleteMyselfNext) delete socketDescriptor;
+  if (!areInRecursiveCall) {
+    socketDescriptor->fAreInReadHandlerLoop = False;
+    if (socketDescriptor->fDeleteMyselfNext) delete socketDescriptor;
+  }
 }
 
 Boolean SocketDescriptor::tcpReadHandler1(int mask) {
