@@ -13,14 +13,15 @@ You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2019, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2024, Live Networks, Inc.  All rights reserved
 // A test program that streams a WAV audio file via RTP/RTCP
 // main program
 
 #include "liveMedia.hh"
-#include "GroupsockHelper.hh"
 
 #include "BasicUsageEnvironment.hh"
+#include "announceURL.hh"
+#include "GroupsockHelper.hh"
 
 // To convert 16-bit samples to 8-bit u-law ("u" is the Greek letter "mu")
 // encoding, before streaming, uncomment the following line:
@@ -162,8 +163,9 @@ void play() {
   }
 
   // Create 'groupsocks' for RTP and RTCP:
-  struct in_addr destinationAddress;
-  destinationAddress.s_addr = chooseRandomIPv4SSMAddress(*env);
+  struct sockaddr_storage destinationAddress;
+  destinationAddress.ss_family = AF_INET;
+  ((struct sockaddr_in&)destinationAddress).sin_addr.s_addr = chooseRandomIPv4SSMAddress(*env);
   // Note: This is a multicast address.  If you wish instead to stream
   // using unicast, then you should use the "testOnDemandRTSPServer" demo application,
   // or the "LIVE555 Media Server" - not this application - as a model.
@@ -212,10 +214,7 @@ void play() {
 	   "Session streamed by \"testWAVAudiotreamer\"", True/*SSM*/);
   sms->addSubsession(PassiveServerMediaSubsession::createNew(*sessionState.sink, sessionState.rtcpInstance));
   sessionState.rtspServer->addServerMediaSession(sms);
-
-  char* url = sessionState.rtspServer->rtspURL(sms);
-  *env << "Play this stream using the URL \"" << url << "\"\n";
-  delete[] url;
+  announceURL(sessionState.rtspServer, sms);
 
   // Finally, start the streaming:
   *env << "Beginning streaming...\n";
